@@ -5,11 +5,12 @@ use swc_common::{collections::AHashSet, Mark, SyntaxContext};
 use swc_ecma_ast::*;
 use swc_ecma_utils::{find_ids, Id};
 use swc_ecma_visit::{as_folder, noop_visit_mut_type, Fold, VisitMut, VisitMutWith};
+use tracing::{debug, span, Level};
 
 #[cfg(test)]
 mod tests;
 
-const LOG: bool = false;
+const LOG: bool = false && cfg!(debug_assertions);
 
 /// See [resolver_with_mark] for docs.
 pub fn resolver() -> impl 'static + Fold + VisitMut {
@@ -119,6 +120,14 @@ impl<'a> Scope<'a> {
             declared_types: Default::default(),
         }
     }
+
+    fn is_declared(&self, symbol: &JsWord) -> bool {
+        if self.declared_symbols.contains(symbol) {
+            return true;
+        }
+
+        self.parent.map_or(false, |p| p.is_declared(symbol))
+    }
 }
 
 /// # Phases
@@ -211,8 +220,8 @@ impl<'a> Resolver<'a> {
     /// Modifies a binding identifier.
     fn modify(&mut self, ident: &mut Ident, kind: Option<VarDeclKind>) {
         if cfg!(debug_assertions) && LOG {
-            eprintln!(
-                "resolver: Binding (type = {}) {}{:?} {:?}",
+            debug!(
+                "Binding (type = {}) {}{:?} {:?}",
                 self.in_type,
                 ident.sym,
                 ident.span.ctxt(),
@@ -233,7 +242,7 @@ impl<'a> Resolver<'a> {
             } else {
                 let span = ident.span.apply_mark(mark);
                 if cfg!(debug_assertions) && LOG {
-                    eprintln!("\t-> {:?}", span.ctxt());
+                    debug!("\t-> {:?}", span.ctxt());
                 }
                 span
             };
@@ -317,7 +326,7 @@ impl<'a> Resolver<'a> {
         } else {
             let span = ident.span.apply_mark(mark);
             if cfg!(debug_assertions) && LOG {
-                eprintln!("\t-> {:?}", span.ctxt());
+                debug!("\t-> {:?}", span.ctxt());
             }
             span
         };
@@ -615,6 +624,12 @@ impl<'a> VisitMut for Resolver<'a> {
     }
 
     fn visit_mut_expr(&mut self, expr: &mut Expr) {
+        let _span = if LOG {
+            Some(span!(Level::ERROR, "visit_mut_expr").entered())
+        } else {
+            None
+        };
+
         self.in_type = false;
         let old = self.ident_type;
         self.ident_type = IdentType::Ref;
@@ -726,8 +741,8 @@ impl<'a> VisitMut for Resolver<'a> {
                 let Ident { span, sym, .. } = i;
 
                 if cfg!(debug_assertions) && LOG {
-                    eprintln!(
-                        "resolver: IdentRef (type = {}) {}{:?}",
+                    debug!(
+                        "IdentRef (type = {}) {}{:?}",
                         self.in_type,
                         sym,
                         span.ctxt()
@@ -742,12 +757,12 @@ impl<'a> VisitMut for Resolver<'a> {
                     let span = span.apply_mark(mark);
 
                     if cfg!(debug_assertions) && LOG {
-                        eprintln!("\t -> {:?}", span.ctxt());
+                        debug!("\t -> {:?}", span.ctxt());
                     }
                     i.span = span;
                 } else {
                     if cfg!(debug_assertions) && LOG {
-                        eprintln!("\t -> Unresolved");
+                        debug!("\t -> Unresolved");
                     }
 
                     let mark = {
@@ -771,7 +786,7 @@ impl<'a> VisitMut for Resolver<'a> {
                     let span = span.apply_mark(mark);
 
                     if cfg!(debug_assertions) && LOG {
-                        eprintln!("\t -> {:?}", span.ctxt());
+                        debug!("\t -> {:?}", span.ctxt());
                     }
 
                     i.span = span;
@@ -917,8 +932,20 @@ impl<'a> VisitMut for Resolver<'a> {
     }
 
     fn visit_mut_stmts(&mut self, stmts: &mut Vec<Stmt>) {
+        let _span = if LOG {
+            Some(span!(Level::ERROR, "visit_mut_stmts").entered())
+        } else {
+            None
+        };
+
         // Phase 1: Handle hoisting
         {
+            let _span = if LOG {
+                Some(span!(Level::ERROR, "hoist").entered())
+            } else {
+                None
+            };
+
             let mut hoister = Hoister {
                 resolver: self,
                 kind: None,
@@ -1386,6 +1413,21 @@ impl VisitMut for Hoister<'_, '_> {
         if self.catch_param_decls.contains(&node.ident.sym) {
             return;
         }
+        let _span = if LOG {
+            Some(span!(Level::ERROR, "Hoister.visit_mut_fn_decl").entered())
+        } else {
+            None
+        };
+
+        if self.in_block {
+            // If we are in nested block, and variable named `foo` is declared, we should
+            // ignore function foo while handling upper scopes.
+            let i = node.ident.clone();
+
+            if self.resolver.current.is_declared(&i.sym) {
+                return;
+            }
+        }
 
         self.resolver.in_type = false;
         self.resolver
@@ -1395,6 +1437,9 @@ impl VisitMut for Hoister<'_, '_> {
     #[inline]
     fn visit_mut_function(&mut self, _: &mut Function) {}
 
+    #[inline]
+    fn visit_mut_ts_module_block(&mut self, _: &mut TsModuleBlock) {}
+
     #[inline]
     fn visit_mut_param(&mut self, _: &mut Param) {}
 
